#!/bin/sh

set -e

#
# compile:
#
make clean
make

#
# remove previus results:
#
rm -f output/*.out output/*.fits

#
# run tests:
#
ininphot=100
firstrun=1

##
## Default values from CO data file: C=3.3e-11 (for T=30K) A=7.2e-08
##

##
## The 'default' run (data point is included in all subsequent simulation series):
##
A=1e-7
C=1e-11
if [ $firstrun == 1 ]
then
    dogridding=1
    firstrun=0
else dogridding=0
fi

echo "... processing run C = $C cm^3 s^-1, A = $A s^-1"
./lime.x --dogridding $dogridding --doraytracing 0 --ininphot $ininphot --settransitionprobabilities 1 --C $C --A $A


##
## The 'black' series:
##
A=1e-7
for C in 1e-3 1e-4 1e-5 1e-6 1e-7 1e-8 1e-9 1e-10 1e-12 1e-13 1e-14 1e-15 1e-16 1e-17 1e-18 1e-19
do

    if [ $firstrun == 1 ]
    then
    dogridding=1
    firstrun=0
    else dogridding=0
    fi

    echo "... processing run C = $C cm^3 s^-1, A = $A s^-1"
    ./lime.x --dogridding $dogridding --doraytracing 0 --ininphot $ininphot --settransitionprobabilities 1 --C $C --A $A

done



##
## The 'green' series:
##
C=1e-11
for A in 1e-1 1e-2 1e-3 1e-4 1e-5 1e-6 1e-8 1e-9 1e-10 1e-11 1e-12 1e-13 1e-14 1e-15
do

    if [ $firstrun == 1 ]
    then
    dogridding=1
    firstrun=0
    else dogridding=0
    fi

    echo "... processing run C = $C cm^3 s^-1, A = $A s^-1"
    ./lime.x --dogridding $dogridding --doraytracing 0 --ininphot $ininphot --settransitionprobabilities 1 --C $C --A $A

done



##
## The 'red' series:
##
C=1e-11
AforB=1e-7
for A in 1e-1 1e-2 1e-3 1e-4 1e-5 1e-6 1e-8 1e-9 1e-10 1e-11 1e-12 1e-13 1e-14 1e-15
do

    if [ $firstrun == 1 ]
    then
    dogridding=1
    firstrun=0
    else dogridding=0
    fi

    echo "... processing run C = $C cm^3 s^-1, A = $A s^-1, AforB = $AforB s^-1"
    ./lime.x --dogridding $dogridding --doraytracing 0 --ininphot $ininphot --settransitionprobabilities 1 --C $C --A $A --AforB $AforB

done


##
## The 'blue' series:
##
C=1e-11
A=1e-7
for AforB in 1e+0 1e-1 1e-2 1e-3 1e-4 1e-5 1e-6 1e-8 1e-9 1e-10 1e-11 1e-12 1e-13 1e-14 1e-15
do

    if [ $firstrun == 1 ]
    then
    dogridding=1
    firstrun=0
    else dogridding=0
    fi

    echo "... processing run C = $C cm^3 s^-1, A = $A s^-1, AforB = $AforB s^-1"
    ./lime.x --dogridding $dogridding --doraytracing 0 --ininphot $ininphot --settransitionprobabilities 1 --C $C --A $A --AforB $AforB

done
