#include "lime.h"
#if LINEPOLARIZATION == ON
#include "linepolarization.h"
#endif

#include <gsl/gsl_sort.h>
#include <gsl/gsl_statistics.h>

// ***********
// ** Setup **
// ***********
char *modelfolder;

double Rcloud = 0.1*PC;

#if LINEPOLARIZATION == ON
int DebugLinePolarization = 0;
#endif
#ifdef INTERNALSOURCES
int DebugInternalSources = 1;
#endif

/*****************************************************************/
void
input(inputPars *par, image *img){

	modelfolder = malloc(100*sizeof(char));
	strcpy(modelfolder, ".");
	
#if LINEPOLARIZATION == ON
	if(DebugLinePolarization) DebugLinePolarizationFile = fopen("debug_LinePolarization.out", "w");
#endif
#ifdef INTERNALSOURCES
	if(DebugInternalSources) DebugInternalSourcesFile = fopen("debug_InternalSources.out", "w");
#endif
#ifdef ROLF
	ROLFout = fopen("debug_ROLF.out", "w");
#endif

	par->radius		   = Rcloud;
	par->minScale	   = 100*AU;
	par->pIntensity    = 3000;
	par->sinkPoints    = 3000;
    par->dust          = "../../input/jena_thin_e5.tab";
    par->moldatfile[0] = "../../input/TwoLevelCO.dat";
	par->sampling	   = 0;
#if LINEPOLARIZATION == ON
	par->blend         = 1;
#else
	par->blend         = 0;	
#endif
	temperature(1, 1, 1, &(par->tcmb));

	char *popsfilename;
	popsfilename = malloc(255*sizeof(char));
#if LINEPOLARIZATION == ON
	sprintf(popsfilename, "%s/Output/fraction=%.2f_pops.out", modelfolder, uservar1);
#else
	sprintf(popsfilename, "./test-pops.out");
#endif
	par->outputfile    = popsfilename;

	
	int i;
	i = 0;
	//
	// Ray-Trace:
	//
#if LINEPOLARIZATION == ON
	img[i].unit			= 13;
	img[i].filename		= malloc(255*sizeof(char));
	sprintf(img[i].filename, "%s/Output/gamma=%.0f_Px.fits", modelfolder, uservar4);
	i++;

	img[i].unit			= 14;
	img[i].filename		= malloc(255*sizeof(char));
	sprintf(img[i].filename, "%s/Output/gamma=%.0f_Py.fits", modelfolder, uservar4);
	i++;

	img[i].unit			= 15;
	img[i].filename		= malloc(255*sizeof(char));
	sprintf(img[i].filename, "%s/Output/gamma=%.0f_Pz.fits", modelfolder, uservar4);
	i++;

	img[i].unit			= 16;
	img[i].filename		= malloc(255*sizeof(char));
	sprintf(img[i].filename, "%s/Output/gamma=%.0f_Taux.fits", modelfolder, uservar4);
	i++;

	img[i].unit			= 17;
	img[i].filename		= malloc(255*sizeof(char));
	sprintf(img[i].filename, "%s/Output/gamma=%.0f_Tauy.fits", modelfolder, uservar4);
	i++;

	img[i].unit			= 18;
	img[i].filename		= malloc(255*sizeof(char));
	sprintf(img[i].filename, "%s/Output/gamma=%.0f_Tauz.fits", modelfolder, uservar4);
	i++;
	
	img[i].unit			= 1;
	img[i].filename		= malloc(255*sizeof(char));
	sprintf(img[i].filename, "%s/Output/gamma=%.0f_Itot.fits", modelfolder, uservar4);
	i++;
	
	int j;
	double TotalFieldInArcsec;
	
	for(j = 0; j < i; j++){
		
		img[j].trans		= 0;												// zero-indexed radiative transition number
		img[j].theta		= 0;												 
		img[j].phi          = 0;
		img[j].nchan		= 1;												// Number of channels
		img[j].velres		= 50.;												// Channel resolution in m/s
		img[j].distance		= 10*PC;											// source distance in m
		TotalFieldInArcsec = 2.0 * atan(par->radius / img[j].distance) * 180.0/PI * 360.0;
		
		//
		// ray-trace the center of the cloud (inner permille):
		//
		img[j].pxls			= 2;												// Pixels per dimension
		img[j].imgres		= 0.001 * 10*TotalFieldInArcsec / (double) img[j].pxls;		// Resolution in arc seconds per pixel
	}
#else
	img[i].trans		= 0;												// zero-indexed radiative transition number
	img[i].unit			= 1;												// 0:Kelvin 1:Jansky/pixel 2:SI 3:Lsun/pixel
	img[i].filename		= malloc(255*sizeof(char));
	sprintf(img[i].filename, "test.fits");
	img[i].theta		= inclination;										// 0: face-on, pi/2: edge-on along y
	img[i].nchan		= 60;												// Number of channels
	img[i].velres		= 50.;												// Channel resolution in m/s
	img[i].pxls			= pxls;												// Pixels per dimension
	img[i].distance		= 10*PC;											// Source distance in m
	img[i].imgres		= 100.0/pxls *2 *100.0* Rcore/20626.5 /10.;			// Resolution in arc seconds per pixel
	img[i].source_vel	= 0;												// Source velocity in m/s
	img[i].phi          = 0;                                                // 
	i++;
	
	img[i].trans		= 0;												// zero-indexed radiative transition number
	img[i].unit			= 4;												// 0:Kelvin 1:Jansky/pixel 2:SI 3:Lsun/pixel
	img[i].filename		= malloc(255*sizeof(char));
	sprintf(img[i].filename, "test-tau.fits");
	img[i].theta		= inclination;										// 0: face-on, pi/2: edge-on
	img[i].nchan		= 60;												// Number of channels
	img[i].velres		= 50.;												// Channel resolution in m/s
	img[i].pxls			= pxls;												// Pixels per dimension
	img[i].distance		= 10*PC;											// Source distance in m
	img[i].imgres		= 100.0/pxls *2 *100.0* Rcore/20626.5 /10.;			// Resolution in arc seconds per pixel
	img[i].source_vel	= 0;												// Source velocity in m/s
	img[i].phi          = 0;                                                // 
#endif

	
//
// specify the zero-indexed lower transition levels of the radiative transitions, which should be treated during line polarization,
//  e.g for J=1 --> J=0 specify a 0
//  or  for J=2 --> J=1 specify a 1 etc.
//
#if LINEPOLARIZATION == ON
	linepolarization_NSplittedLevels = 1;
	linepolarization_SplittedLevels = malloc(linepolarization_NSplittedLevels * sizeof(int));
	linepolarization_SplittedLevels[0] = 0;
#endif
}

/*****************************************************************/

/* density[i] is the number density of the i'th collision partner. */
void
density(double x, double y, double z, double *density){	
	density[0]  = 1e+2; // in [cm^-3]
	density[0] *= 1e+6; // in [m^-3]
}


/* temperature[0] is the kinetic gas temperature.
 * temperature[1] is the dust temperature.
 * temperature[1] is optional. If left out, LIME will assume that t_dust=t_gas. */
void
temperature(double x, double y, double z, double *temperature){
	temperature[0] = 30.0;
}

/* abundance[i] gives the fractional abundance of the i'th molecular species given by the i'th moldatfile. */
void
abundance(double x, double y, double z, double *abundance){
	abundance[0] = 1e-4; // CO to H2
}

/* This function gives the value of the Doppler b- parameter (1/e half width of the line profile) in m s^-1. */
void
doppler(double x, double y, double z, double *doppler){
	*doppler = 200.;
}

/* vel holds the x, y, and z components of the velocity vectors. */
double signof(double a) { return (a == 0) ? 0 : (a<0 ? -1 : 1); }
void
velocity(double x, double y, double z, double *vel){
	
	double r, theta, phi, vr;
    double v0low, v0high;
    
	//
	// Spherical coordinates:
	//
	r     = sqrt(x*x + y*y + z*z); // radius
	theta = acos(z/r);             // inclination
	phi   = atan2(y,x);            // azimuth
	
	//
	// Velocities:
	//
	vr     = 10.0; // in km/s
    v0low  =  1.0; // in km/s
    v0high = 10.0; // in km/s

	//
	// convert from km/s to m/s:
	//
	vr     *= 1000.0;
    v0low  *= 1000.0;
    v0high *= 1000.0;
    
	//
	// add gradient:
	//
	vr *= (Rcloud-r)/Rcloud;
	
	//
	// convert to Cartesian coordinates:
	//
	vel[0] = uservar1 * vr * sin(theta) * cos(phi);
	vel[1] = uservar1 * vr * sin(theta) * sin(phi);
	vel[2] = uservar1 * vr * cos(theta);
    
    //
    // add contraction (faster along z):
    //
    vel[0] += -(1.0 - uservar1) * v0low  * signof(x)*(Rcloud-fabs(x))/Rcloud;
    vel[1] += -(1.0 - uservar1) * v0low  * signof(y)*(Rcloud-fabs(y))/Rcloud;
    vel[2] += -(1.0 - uservar1) * v0high * signof(z)*(Rcloud-fabs(z))/Rcloud;    
}

#if LINEPOLARIZATION == ON
// definition in SI units: B-field in Tesla = 10^4 Gauss
void
magneticfield(double x, double y, double z, double *b){

	double b0 = 1e-10; // in [T] = [10^4 G] = [10^7 mG] = [10^10 microG]
	
	//
	// Radial magnetic field:
	//
	b[0] = x;
	b[1] = y;
	b[2] = z;
	vectoralgebra_Normalize(b);

	//
	// Contribution of the radial magnetic field:
	//
	b[0] *= uservar1;
	b[1] *= uservar1;
	b[2] *= uservar1;
	
	//
	// Add the contribution of the 1D directed magnetic field:
	//
	b[2] += (1.0 - uservar1);
	
	//
	// Scale to default magnetic field strength:
	//
	b[0] *= b0;
	b[1] *= b0;
	b[2] *= b0;
	
}
#endif

/******************************************************************/
